rm(list=ls())

#chargement des packages
library(reshape2)
library(maptools)
library(RColorBrewer)

#recherche des flux dominants

matmig <- read.csv("matmig_1961_1965.csv", sep=";", 
                   header=TRUE, stringsAsFactors=FALSE)

mat <- as.matrix(matmig[,-1])
row.names(mat) <- matmig[,1]
mat

Oi <- apply(mat, 1, sum)  # somme (en ligne) des départs
Dj <- apply(mat, 2, sum) # sommes (en colonne) des arrivées
FMaj <- apply(mat, 1, max) # flux majeur pour chaque i
DMaj <- apply(mat, 1, which.max)  # destination du flux majeur pour 
                                  #chaque i
NMaj <- names(mat[,1])[DMaj]  # nom de la destination majeure
FDom <- Dj < Dj[DMaj]  # renvoie T/F => i est-il dominé par j ?

FluxDom <- as.data.frame(cbind(NMaj, FMaj , FDom)) 
FluxDom <- FluxDom[FluxDom$FDom == T,]

FluxDom$i <- row.names(FluxDom)
colnames(FluxDom) <- c("j","Fij","T", "i")
FluxDom

#cartographie

#chargement du fonds de carte
fdc <- readShapeSpatial("tch10.shp")

#calcul des centroides et visualisation des codes
centres <- cbind(fdc@data[,"CODE"], as.data.frame(coordinates(fdc)))
colnames(centres) <- c("CODE","X","Y")
head(centres, 4)

#ajout des coordonnées au fichier des flux
coordi <- centres
names(coordi) <- c("i","Xi","Yi")
tabflow <- merge(FluxDom, coordi, by="i", all.x=TRUE)

coordj <- centres
names(coordj) <- c("j","Xj","Yj")
tabflow <- merge(tabflow, coordj, by="j", all.x=TRUE)

#calcul des distances euclidiennes
tabflow$Dij <- round(sqrt((tabflow$Xi-tabflow$Xj)**2
                          +(tabflow$Yi-tabflow$Yj)**2)/1000, 1)
head(tabflow, 4)

#cartographie d'une ou plusieurs couches de carte (facultatif)
dev.off()
plot(fdc, col="lightyellow", border="gray", lwd=1)

fdc2 <- readShapeSpatial("tch2.shp")
plot(fdc2, border="black", lwd=1.5, add=TRUE)

#cartographie des flux
arrows(tabflow$Xi, tabflow$Yi, tabflow$Xj, tabflow$Yj, 
       col="red", lwd=3, code = 2)

#ajout des codes (facultatif)
text(centres$X, centres$Y, centres$CODE, col="blue", cex=0.8)

#ajout du titre et du sous-titre (facultatif)
title(main="Flux dominants")

#flux majeurs

rm(list=ls())

#importation et transformation d'une matrice de flux

#chargement d'une matrice de flux
tab <- read.table("matmig_1961_1965.csv",
                  header=TRUE,
                  sep=";",
                  stringsAsFactors=FALSE,
                  encoding="UTF-8")
str(tab)

mat <- as.matrix(tab[,-1])
row.names(mat) <- tab[,1]

#passage en rang

FijR <- t(apply(mat, 1, rank)) #rang des origines
FijR

#passage au format origine - destination - flux
tabflow <- melt(FijR)
names(tabflow)=c("i","j","Rij")
head(tabflow, 4)

#import du fond de carte : ajout des coordonnées et calcul des distances 

#chargement du fonds de carte
fdc <- readShapeSpatial("tch10.shp")

#calcul des centroides et visualisation des codes
centres <- cbind(fdc@data[,"CODE"], as.data.frame(coordinates(fdc)))
colnames(centres) <- c("CODE","X","Y")
head(centres, 4)

#ajout des coordonnées au fichier des flux
coordi <- centres
names(coordi) <- c("i","Xi","Yi")
tabflow <- merge(tabflow, coordi, by="i", all.x=TRUE)

coordj <- centres
names(coordj) <- c("j","Xj","Yj")
tabflow <- merge(tabflow, coordj, by="j", all.x=TRUE)

#calcul des distances euclidiennes
tabflow$Dij <- round(sqrt((tabflow$Xi-tabflow$Xj)**2
                          +(tabflow$Yi-tabflow$Yj)**2)/1000, 1)
head(tabflow, 4)

#choix du seuil

sel <- tabflow[tabflow$Rij > 8,] #2 flux majeurs
sel
dim(sel)

#retouche des coordonnées pour éloigner les flèches des centres
coeff <- 0.15
sel$XXi <- sel$Xi+coeff*(sel$Xj-sel$Xi)
sel$XXj <- sel$Xj-coeff*(sel$Xj-sel$Xi)
sel$YYi <- sel$Yi+coeff*(sel$Yj-sel$Yi)
sel$YYj <- sel$Yj-coeff*(sel$Yj-sel$Yi)

#cartographie d'une ou plusieurs couches de carte (facultatif)
dev.off()
plot(fdc, col="lightyellow", border="gray", lwd=1)

fdc2 <- readShapeSpatial("tch2.shp")
plot(fdc2, border="black", lwd=1.5, add=TRUE)

#cartographie des flux
arrows(sel$XXi, sel$YYi, sel$XXj, sel$YYj, col="red", lwd=3)

#ajout des codes (facultatif)
text(centres$X, centres$Y, centres$CODE, col="blue", cex=0.8)

#ajout du titre et du sous-titre (facultatif)
title(main="Flux majeurs")

#flux majeurs - variante flux > x % du flux total 

#volume total

#somme des départs par origine
O <- apply(mat, 1, sum) 
#volume global
V <- sum(O)
V
#passage en %

mat2 <- (mat/V)*100

#sélection et cartographie
#controle - flux >2% du total

#passage au format origine - destination - flux
tabflow <- melt(mat2)
names(tabflow)=c("i","j","PCij")
head(tabflow, 4)

#import du fond de carte : ajout des coordonnées et calcul des distances 

#chargement du fonds de carte
fdc <- readShapeSpatial("tch10.shp")

#calcul des centroides et visualisation des codes
centres <- cbind(fdc@data[,"CODE"], as.data.frame(coordinates(fdc)))
colnames(centres) <- c("CODE","X","Y")
head(centres, 4)

#ajout des coordonnées au fichier des flux
coordi <- centres
names(coordi) <- c("i","Xi","Yi")
tabflow <- merge(tabflow, coordi, by="i", all.x=TRUE)

coordj <- centres
names(coordj) <- c("j","Xj","Yj")
tabflow <- merge(tabflow, coordj, by="j", all.x=TRUE)

#calcul des distances euclidiennes
tabflow$Dij <- round(sqrt((tabflow$Xi-tabflow$Xj)**2
                          +(tabflow$Yi-tabflow$Yj)**2)/1000, 1)
head(tabflow, 4)

#choix du seuil

sel <- tabflow[tabflow$PCij >2,] #2% et plus
sel
dim(sel)

#retouche des coordonnées pour éloigner les flèches des centres
coeff <- 0.15
sel$XXi <- sel$Xi+coeff*(sel$Xj-sel$Xi)
sel$XXj <- sel$Xj-coeff*(sel$Xj-sel$Xi)
sel$YYi <- sel$Yi+coeff*(sel$Yj-sel$Yi)
sel$YYj <- sel$Yj-coeff*(sel$Yj-sel$Yi)

#cartographie d'une ou plusieurs couches de carte (facultatif)
dev.off()
plot(fdc, col="lightyellow", border="gray", lwd=1)

fdc2 <- readShapeSpatial("tch2.shp")
plot(fdc2, border="black", lwd=1.5, add=TRUE)

#cartographie des flux
arrows(sel$XXi, sel$YYi, sel$XXj, sel$YYj, col="red", lwd=3)

#ajout des codes (facultatif)
text(centres$X, centres$Y, centres$CODE, col="blue", cex=0.8)

#ajout du titre et du sous-titre (facultatif)
title(main="Flux majeurs (> 2% du total)")

